import tempfile

from BTrees.Length import Length

from five import grok

from zope import schema
from zope.annotation.interfaces import IAnnotations
from zope.interface import implements
from zope.lifecycleevent.interfaces import IObjectAddedEvent

from plone.autoform import directives as form
from plone.dexterity.content import Item
from plone.dexterity.schema import DexteritySchemaPolicy
from plone.directives.form import default_value
from plone.namedfile.field import NamedBlobFile
from plone.supermodel import model
from plone.app.contenttypes.interfaces import IFile
from Products.CMFPlone.utils import base_hasattr

from . import _


class IDmsFile(model.Schema, IFile):
    """Schema for DmsFile"""
    title = schema.TextLine(
        title=_(u'Version number'),
        required=False
    )
    form.mode(title='hidden')

    model.primary('file')
    file = NamedBlobFile(
        title=_(u"File"),
        required=True,
    )

    label = schema.TextLine(
        title=_(u'Label'),
        required=False,
    )


class DmsFile(Item):
    """DmsFile"""
    implements(IDmsFile)
    __ac_local_roles_block__ = True

    incomingmail = False

    def Title(self):
        if self.incomingmail:
            return _(u"Incoming mail")
        elif base_hasattr(self, 'signed') and self.signed:
            return _(u"Signed version")
        else:
            return self.title


class DmsFileSchemaPolicy(DexteritySchemaPolicy):
    """Schema Policy for DmsFile"""

    def bases(self, schemaName, tree):
        return (IDmsFile, )


class IDmsAppendixFile(model.Schema, IFile):
    """Schema for DmsAppendixFile"""
    model.primary('file')
    file = NamedBlobFile(
        title=_(u"File"),
        required=True,
    )


class DmsAppendixFile(Item):
    """DmsAppendixFile"""
    implements(IDmsAppendixFile)
    __ac_local_roles_block__ = True


class DmsAppendixFileSchemaPolicy(DexteritySchemaPolicy):
    """Schema Policy for DmsAppendixFile"""

    def bases(self, schemaName, tree):
        return (IDmsAppendixFile, )


@default_value(field=IDmsFile['title'])
def titleDefaultValue(data):
    container = data.context
    annotations = IAnnotations(container)
    if 'higher_version' not in annotations:
        version_number = 1
    else:
        version_number = annotations['higher_version'].value + 1
    return unicode(version_number)


@grok.subscribe(IDmsFile, IObjectAddedEvent)
def update_higher_version(context, event):
    container = context.getParentNode()
    annotations = IAnnotations(container)
    if 'higher_version' not in annotations:
        annotations['higher_version'] = Length()
    annotations['higher_version'].change(1)



# -*- coding: utf8 -*-

from collective.dms.basecontent.source import RecipientGroupsVocabulary
from collective.dms.basecontent.source import TreatingGroupsVocabulary
from collective.dms.basecontent.testing import DMS_TESTS_PROFILE_FUNCTIONAL
from ecreall.helpers.testing.base import BaseTest
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID

import unittest

from plone.dexterity.utils import createContentInContainer


class TestDmsdocument(unittest.TestCase, BaseTest):
    """Tests adapters"""

    layer = DMS_TESTS_PROFILE_FUNCTIONAL

    def setUp(self):
        super(TestDmsdocument, self).setUp()
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.doc = createContentInContainer(self.portal, 'dmsdocument', **{'title': 'Doc 1'})

    def test_RecipientGroupsVocabulary(self):
        voc_inst = RecipientGroupsVocabulary()
        voc_ids = [i.token for i in voc_inst(self.portal).__iter__()]
        self.assertEqual(set(voc_ids), set(['Administrators', 'Reviewers', 'Site Administrators']))
        voc_ids = [i.token for i in voc_inst(self.portal).search('user')]
        self.assertEqual(set(voc_ids), set(['test_user_1_', 'AuthenticatedUsers']))

    def test_TreatingGroupsVocabulary(self):
        voc_inst = TreatingGroupsVocabulary()
        voc_ids = [i.token for i in voc_inst(self.portal).__iter__()]
        self.assertEqual(set(voc_ids), set(['Administrators', 'Reviewers', 'Site Administrators']))
        voc_ids = [i.token for i in voc_inst(self.portal).search('user')]
        self.assertEqual(set(voc_ids), set(['test_user_1_', 'AuthenticatedUsers']))

    def test_getmainfiles(self):
        self.assertListEqual(self.doc.get_mainfiles(), [])
        file1 = createContentInContainer(self.doc, 'dmsmainfile', **{'title': 'MF 1'})
        createContentInContainer(self.doc, 'dmsappendixfile', **{'title': 'AF 1'})
        self.assertListEqual(self.doc.get_mainfiles(), [file1])
        file2 = createContentInContainer(self.doc, 'dmsmainfile', **{'title': 'MF 2'})
        self.assertListEqual(self.doc.get_mainfiles(), [file1, file2])
        self.doc.moveObjectsByDelta([file2.id], -2)
        self.assertListEqual(self.doc.get_mainfiles(), [file2, file1])



def setup_documentviewer(portal):
    from collective.documentviewer.settings import GlobalSettings
    dv_settings = GlobalSettings(portal)
    dv_settings.auto_layout_file_types = ('pdf', 'ppt', 'word', 'rft')
    dv_settings.auto_convert = True
    dv_settings.show_sidebar = False
    dv_settings.show_search = False


def importFinalSteps(context):
    """Import all final steps.
    """
    marker = context.readDataFile('collective_dms_basecontent_marker.txt')
    if marker is None:
        return

    site = context.getSite()
    setup_documentviewer(site)



from AccessControl import getSecurityManager
from zope.interface import implements, implementer
from zope.cachedescriptors.property import CachedProperty
from zope.component import adapter, getUtility
from zope.intid.interfaces import IIntIds

from zc.relation.interfaces import ICatalog

from z3c.form.interfaces import IFormLayer, IFieldWidget
from z3c.form.widget import FieldWidget
from z3c.relationfield.interfaces import IRelationList
from z3c.relationfield.schema import RelationChoice, RelationList

from plone.formwidget.contenttree.widget import MultiContentTreeWidget
from plone.formwidget.contenttree import ObjPathSourceBinder

from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

class IRelatedDocs(IRelationList):
    """"""

class RelatedDocsWidget(MultiContentTreeWidget):
    display_template = ViewPageTemplateFile('related-docs-display.pt')

    def __init__(self, display_backrefs, request):
        self.display_backrefs = display_backrefs
        super(RelatedDocsWidget, self).__init__(request)

    def get_url(self, v):
        return self.request.physicalPathToURL(v)

    def get_label(self, v):
        term = self.terms.getTermByToken(v)
        return term.title

    def update(self):
        super(RelatedDocsWidget, self).update()
        if self.mode == 'display':
            if self.tuples and not self.value:
                # hack to not have the 'empty' class added to the field
                self.value = ["don't hide field"]

    @CachedProperty
    def tuples(self):
        refs = [(self.get_url(x), self.get_label(x)) for x in self.value]
        if self.display_backrefs:
            intids = getUtility(IIntIds)
            catalog = getUtility(ICatalog)
            try:
                doc_intid = intids.getId(self.context)
            except KeyError:
                pass
            else:
                sm = getSecurityManager()
                for ref in catalog.findRelations({'to_id': doc_intid}):
                    obj = ref.from_object
                    if not sm.checkPermission('View', obj):
                        continue
                    url = self.get_url(ref.from_path)
                    tp = (url, obj.Title())
                    if tp not in refs:
                        refs.append(tp)
        return refs


@adapter(IRelatedDocs, IFormLayer)
@implementer(IFieldWidget)
def RelatedDocsFieldWidget(field, request):
    return FieldWidget(field, RelatedDocsWidget(field.display_backrefs, request))


class RelatedDocs(RelationList):
    implements(IRelatedDocs)

    def __init__(self, portal_types=None, display_backrefs=False, object_provides=None, **kwargs):
        self.display_backrefs = display_backrefs
        kw = dict()
        if portal_types:
            kw['portal_type'] = portal_types
        if object_provides:
            kw['object_provides'] = object_provides
        RelationList.__init__(self,
                        value_type=RelationChoice(
                            title=u'',
                            source=ObjPathSourceBinder(**kw)),
                        **kwargs)



import json
from zope.component import getMultiAdapter
from collective.documentviewer.views import DocumentViewerView
from plone.dexterity.browser.view import DefaultView
from plone.dexterity.browser.edit import DefaultEditForm
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile


class VersionViewerView(DocumentViewerView):
    pass


class JSONVersionViewerView(DocumentViewerView):
    def index(self):
        self.request.response.setHeader('Content-Type', 'application/json')
        return json.dumps(self.dv_data())


class DmsDocumentView(DefaultView):
    def update(self):
        super(DmsDocumentView, self).update()
        self.portal_url = getMultiAdapter((self.context, self.request),
                                          name="plone_portal_state").portal_url()
        self.dvstatic = "%s/++resource++dv.resources" % (self.portal_url)


class DmsDocumentEdit(DefaultEditForm):
    template = ViewPageTemplateFile('templates/dmsdocument_edit.pt')

    def update(self):
        super(DmsDocumentEdit, self).update()
        self.portal_url = getMultiAdapter((self.context, self.request),
                                          name="plone_portal_state").portal_url()
        self.dvstatic = "%s/++resource++dv.resources" % (self.portal_url)



# -*- coding: utf-8 -*-
import logging

from plone import api

logger = logging.getLogger('collective.dms.basecontent: upgrade. ')


def v2(context):
    # install product
    setup = api.portal.get_tool('portal_setup')
    setup.runAllImportStepsFromProfile('profile-dexterity.localrolesfield:default')
    # remove old roles set
    config = {'treating_groups': 'Editor', 'recipient_groups': 'Reader'}
    catalog = api.portal.get_tool('portal_catalog')
    for brain in catalog(portal_type='dmsdocument'):
        obj = brain.getObject()
        for local_principal, local_roles in dict(obj.get_local_roles()).items():
            # a local_role is like (u'Contributor', u'Reviewer'))
            for attr, role in config.items():
                if local_principal in getattr(obj, attr) or []:
                    cleaned_local_roles = list(local_roles)
                    try:
                        cleaned_local_roles.remove(role)
                    except ValueError:
                        # if a role to remove was already removed (???) pass
                        logger.warn("Failed to remove role '%s' for principal '%s' on object '%s'"
                                    % (role, local_principal, '/'.join(obj.getPhysicalPath())))
                    # if there are still some local_roles, use manage_setLocalRoles
                    if cleaned_local_roles:
                        obj.manage_setLocalRoles(local_principal, cleaned_local_roles)
                    else:
                        # either use manage_delLocalRoles
                        obj.manage_delLocalRoles((local_principal, ))



from zope.interface import implementer
import z3c.form.interfaces

from collective.z3cform.chosen.widget import AjaxChosenMultiSelectionWidget


@implementer(z3c.form.interfaces.IFieldWidget)
def AjaxChosenMultiFieldWidget(field, request):
    widget = z3c.form.widget.FieldWidget(field,
        AjaxChosenMultiSelectionWidget(request))
    widget.populate_select = True
    return widget



import datetime

from five import grok
from zope.cachedescriptors.property import CachedProperty
from zope.component import getMultiAdapter
from zope.i18nmessageid import MessageFactory
import z3c.table.table
import z3c.table.column
from z3c.table.interfaces import IBatchProvider

from plone.batching.interfaces import IBatch
from plone import api

PMF = MessageFactory('plone')

grok.templatedir('templates')


class TableViewlet(grok.Viewlet):
    grok.baseclass()
    grok.template('filesviewlet')
    __table__ = NotImplemented
    label = NotImplemented
    noresult_message = NotImplemented

    def update(self):
        self.table = self.__table__(self.context, self.request)
        self.table.viewlet = self
        self.table.update()


class Table(z3c.table.table.Table):
    cssClassEven = u'even'
    cssClassOdd = u'odd'
    cssClasses = {'table': 'listing nosort'}
    sortOn = None
    batchSize = 10000  # not used
    startBatchingAt = 10000  # not used
    batchProviderName = 'plonebatch'

    def batchRows(self):
        # this is not self.rows that is batched, but self.values
        pass

    def updateBatch(self):
        if IBatch.providedBy(self.values):
            self.batchProvider = getMultiAdapter((self.context,
                self.request, self), IBatchProvider,
                name=self.batchProviderName)
            self.batchProvider.update()

    @CachedProperty
    def translation_service(self):
        return api.portal.get_tool('translation_service')

    @CachedProperty
    def wtool(self):
        return api.portal.get_tool('portal_workflow')

    @CachedProperty
    def portal_url(self):
        return api.portal.get().absolute_url()

    def update(self):
        super(Table, self).update()

    def format_date(self, date, long_format=None, time_only=None):
        if date is None:
            return u""

        # If date is a datetime object, isinstance(date, datetime.date)
        # returns True, so we use type here.
        if type(date) == datetime.date:
            date = date.strftime('%Y/%m/%d')
        elif type(date) == datetime.datetime:
            date = date.strftime('%Y/%m/%d %H:%M')

        return self.translation_service.ulocalized_time(
            date,
            long_format=long_format,
            time_only=time_only,
            context=self.context,
            domain='plonelocales',
            request=self.request)

    def renderRow(self, row, cssClass=None):
        from .column import StateColumn, get_value
        isSelected = self.isSelectedRow(row)
        if isSelected and self.cssClassSelected and cssClass:
            cssClass = '%s %s' % (self.cssClassSelected, cssClass)
        elif isSelected and self.cssClassSelected:
            cssClass = self.cssClassSelected
        cells = [self.renderCell(item, col, colspan)
                 for item, col, colspan in row]

        state_column = [x for x in row if isinstance(x[1], StateColumn)]
        if state_column:
            state_column = state_column[0]
            state_value = get_value(state_column[0], 'review_state')
            if state_value:
                cssClass += ' row-state-%s' % state_value

        cssClass = self.getCSSClass('tr', cssClass)
        return u'\n    <tr%s>%s\n    </tr>' % (cssClass, u''.join(cells))



from imio.helpers.content import object_values
from zope import schema
from zope.interface import implements
from zope.schema.fieldproperty import FieldProperty

from plone.autoform import directives as form
from plone.dexterity.content import Container
from plone.dexterity.schema import DexteritySchemaPolicy

from plone.supermodel import model
from collective.dms.basecontent.relateddocs import RelatedDocs
from dexterity.localrolesfield.field import LocalRolesField

from . import _
from .widget import AjaxChosenMultiFieldWidget


class IDmsDocument(model.Schema):
    """Schema for DmsDocument"""

    notes = schema.Text(
        title=_(u"Notes"),
        required=False,
    )

    treating_groups = LocalRolesField(
        title=_(u"Treating groups"),
        required=False,
        value_type=schema.Choice(vocabulary=u'collective.dms.basecontent.treating_groups',)
    )
    # form.widget(treating_groups=AjaxChosenMultiFieldWidget)

    recipient_groups = LocalRolesField(
        title=_(u"Recipient groups"),
        required=False,
        value_type=schema.Choice(vocabulary=u'collective.dms.basecontent.recipient_groups')
    )
    form.widget(recipient_groups=AjaxChosenMultiFieldWidget)

    related_docs = RelatedDocs(
        title=_(u"Related documents"),
        required=False,
        display_backrefs=True,
    )


class DmsDocument(Container):
    """DmsDocument"""
    implements(IDmsDocument)
    # disable local roles inheritance
    __ac_local_roles_block__ = True

    def get_mainfiles(self):
        return object_values(self, ['DmsFile'])


class DmsDocumentSchemaPolicy(DexteritySchemaPolicy):
    """DmsDocument schema policy"""

    def bases(self, schemaName, tree):
        return (IDmsDocument, )



#! -*- coding: utf8 -*-
from setuptools import setup, find_packages

version = '1.9'

long_description = (
    open('README.rst').read()
    + '\n' +
    'Contributors\n'
    '============\n'
    + '\n' +
    open('CONTRIBUTORS.rst').read()
    + '\n' +
    open('CHANGES.rst').read()
    + '\n')

setup(
    name='collective.dms.basecontent',
    version=version,
    description="Base content types for document management system",
    long_description=long_description,
    # Get more strings from
    # http://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Framework :: Plone",
        "Framework :: Plone :: 4.2",
        "Framework :: Plone :: 4.3",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    keywords='document management system dms viewer',
    author='Ecreall, Entrouvert, IMIO',
    author_email='cedricmessiant@ecreall.com',
    url='https://github.com/collective/collective.dms.basecontent',
    download_url='https://pypi.org/project/collective.dms.basecontent',
    license='gpl',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    namespace_packages=['collective', 'collective.dms'],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'setuptools',
        'five.grok',
        'collective.documentviewer',
        'dexterity.localrolesfield',
        'future',
        'imio.helpers>=0.42',
        'pdbpp',
        'plone.api',
        'plone.app.dexterity',
        'plone.directives.form',
        'plone.namedfile',
        'z3c.blobfile',
        'plone.app.contenttypes',
        'plone.app.relationfield',
        'plone.formwidget.contenttree',
        'plone.principalsource',
        'collective.z3cform.chosen',
        'z3c.table>=2.2',
    ],
    extras_require={
        'test': ['plone.app.testing',
                 'ecreall.helpers.testing',
                 'plone.app.vocabularies'
                 ],
    },
    entry_points="""
    # -*- Entry points: -*-
    [z3c.autoinclude.plugin]
    target = plone
    """,
)



from five import grok
from zope.schema.interfaces import IVocabularyFactory
from plone.principalsource.source import PrincipalSourceBinder, PrincipalSource


# By default, we list groups and we can search for users in ajax
class PrincipalSource(PrincipalSource):
    def search_principals(self, groups_first=False, **kw):
        if kw:
            results = self.acl_users.searchPrincipals(groups_first=True, **kw)
        else:
            # if no kw, we have been called from source __iter__ because
            # of Chosen widget populate_select attribute is set to True
            results = self.acl_users.searchGroups()
        return [r for r in results if r.get('groupid', None) != 'AuthenticatedUsers']

    @property
    def _search(self):
        if self.users and self.groups:
#            return self.acl_users.searchPrincipals
            return self.search_principals
        elif self.users:
            return self.acl_users.searchUsers
        elif self.groups:
            return self.acl_users.searchGroups


class PrincipalSourceBinder(PrincipalSourceBinder):

    def __call__(self, context):
        return PrincipalSource(context, self.users, self.groups)


class PrincipalsVocabularyFactory(grok.GlobalUtility):
    """Vocabulary for principals"""
    grok.name('dms.principals')
    grok.implements(IVocabularyFactory)

    def __call__(self, context):
        principals = PrincipalSourceBinder(users=True, groups=True)
        return principals(context)


class TreatingGroupsVocabulary(grok.GlobalUtility):
    """Vocabulary for treating groups"""
    grok.name('collective.dms.basecontent.treating_groups')
    grok.implements(IVocabularyFactory)

    def __call__(self, context):
        principals = PrincipalSourceBinder(users=True, groups=True)
#        principals = queryUtility(IVocabularyFactory, name=u'plone.principalsource.Principals')
        return principals(context)


class RecipientGroupsVocabulary(grok.GlobalUtility):
    """Vocabulary for recipient groups"""
    grok.name('collective.dms.basecontent.recipient_groups')
    grok.implements(IVocabularyFactory)

    def __call__(self, context):
#        principals = queryUtility(IVocabularyFactory, name=u'plone.principalsource.Principals')
        principals = PrincipalSourceBinder(users=True, groups=True)
        return principals(context)



# -*- coding: utf-8 -*-
from five import grok

from zope.interface import Interface
from zope.cachedescriptors.property import CachedProperty
from zope.i18n import translate
from zope.i18nmessageid import MessageFactory

from Products.CMFCore.utils import getToolByName

from collective.dms.basecontent import _
from collective.dms.basecontent.browser import column
from collective.dms.basecontent.browser.table import Table


grok.templatedir('templates')

PMF = MessageFactory('plone')


class BaseTable(Table):

    @CachedProperty
    def values(self):
        portal_catalog = getToolByName(self, 'portal_catalog')
        folder_path = '/'.join(self.context.getPhysicalPath())
        query = {'path': {'query': folder_path},
                 'sort_on': 'getObjPositionInParent',
                 'sort_order': 'ascending'}
        query.update(self.viewlet.contentFilter())
        results = portal_catalog.searchResults(query)
        return results


class VersionsTable(BaseTable):

    cssClasses = {'table': 'listing nosort dv'}


class DmsAppendixTable(VersionsTable):

    def setUpColumns(self):
        columns = super(DmsAppendixTable, self).setUpColumns()
        return [column for column in columns if column.__name__ != 'dms.state']


class BaseTitleColumn(column.TitleColumn):
    grok.name('dms.title')
    grok.adapts(Interface, Interface, BaseTable)


class VersionsTitleColumn(BaseTitleColumn):
    grok.adapts(Interface, Interface, VersionsTable)
    domain = 'collective.dms.basecontent'
    linkCSS = 'version-link'

    def getLinkContent(self, item):
        content = super(VersionsTitleColumn, self).getLinkContent(item)
        return translate(content, domain=self.domain, context=self.request)


class DownloadColumn(column.DownloadColumn):
    grok.name('dms.download')
    grok.adapts(Interface, Interface, VersionsTable)


class EditColumn(column.EditColumn):
    grok.name('dms.edit')
    grok.adapts(Interface, Interface, VersionsTable)


class ExternalEditColumn(column.ExternalEditColumn):
    grok.name('dms.extedit')
    grok.adapts(Interface, Interface, VersionsTable)


class DeleteColumn(column.DeleteColumn):
    grok.name('dms.delete')
    grok.adapts(Interface, Interface, VersionsTable)


class AuthorColumn(column.PrincipalColumn):
    grok.name('dms.author')
    grok.adapts(Interface, Interface, VersionsTable)
    header = _(u"Author")
    weight = 30
    attribute = 'Creator'
    cssClasses = {'th': 'th_header_author', 'td': 'td_cell_author'}


class UpdateColumn(column.DateColumn):
    grok.name('dms.update')
    grok.adapts(Interface, Interface, VersionsTable)
    header = PMF(u"listingheader_modified")
    attribute = 'modified'
    weight = 40
    cssClasses = {'th': 'th_header_modified', 'td': 'td_cell_modified'}


class StateColumn(column.StateColumn):
    grok.name('dms.state')
    grok.adapts(Interface, Interface, BaseTable)
    weight = 50


class VersionLabelColumn(column.LabelColumn):
    grok.name('dms.label')
    grok.adapts(Interface, Interface, VersionsTable)
    attribute = 'label'
    header = _(u"Label")
    weight = 15



from AccessControl import getSecurityManager
from Acquisition import aq_base
from collective.dms.basecontent import _
from five import grok
from html import escape
from plone import api
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.WorkflowCore import WorkflowException
from Products.CMFPlone.utils import safe_unicode
from z3c.table import interfaces
from zope.component import getMultiAdapter
from zope.i18n import translate
from zope.i18nmessageid import MessageFactory

import Missing
import os.path
import plone.api
import z3c.table.column
import z3c.table.table


PMF = MessageFactory('plone')

grok.templatedir('templates')


class Column(z3c.table.column.Column, grok.MultiAdapter):
    grok.baseclass()
    grok.provides(interfaces.IColumn)


def get_value(item, attribute, default=None):
    try:
        value = getattr(aq_base(item), attribute)
        if value is Missing.Value:
            return default
    except AttributeError:
        obj = item.getObject()
        value = getattr(obj, attribute, default)

    if callable(value):
        value = value()

    return value


class DateColumn(Column):
    grok.baseclass()
    attribute = NotImplemented

    def renderCell(self, item):
        value = get_value(item, self.attribute)
        return self.table.format_date(value)


class DateTimeColumn(Column):
    grok.baseclass()
    attribute = NotImplemented

    def renderCell(self, item):
        value = get_value(item, self.attribute)
        return self.table.format_date(value, long_format=True)


def get_user_fullname(username):
    """Get fullname without using getMemberInfo that is slow slow slow..."""
    storage = api.portal.get_tool('acl_users').mutable_properties._storage
    data = storage.get(username, None)
    if data is not None:
        return safe_unicode(data.get('fullname', '') or username)
    else:
        return safe_unicode(username)


class PrincipalColumn(Column):
    grok.baseclass()
    attribute = NotImplemented

    def renderCell(self, item):
        value = get_value(item, self.attribute, default=())

        if not isinstance(value, (list, tuple)):
            value = (value,)

        # gtool = getToolByName(plone.api.portal.get(), 'portal_groups')
        # mtool = getToolByName(plone.api.portal.get(), 'portal_membership')
        principals = []
        for principal_id in value:
            # user = mtool.getMemberById(principal_id)
            # if user is not None:
            #     principals.append(escape(user.getProperty('fullname', None)) or user.getId())
            # else:
            #     group = gtool.getGroupById(principal_id)
            #     if group is not None:
            #         principals.append(escape(group.getProperty('title', None)) or group.getId())
            principals.append(escape(get_user_fullname(principal_id)))

        return u', '.join(principals)


class LinkColumn(z3c.table.column.LinkColumn, Column):
    grok.baseclass()

    def getLinkURL(self, item):
        """Setup link url."""
        if self.linkName is not None:
            return '%s/%s' % (item.getURL(), self.linkName)
        return item.getURL()

    def renderCell(self, item):
        # setup a tag
        return '<a href="%s"%s%s%s>%s</a>' % (
            escape(self.getLinkURL(item)),
            self.getLinkTarget(item),
            self.getLinkCSS(item),
            self.getLinkTitle(item),
            self.getLinkContent(item),  # originally escaped
        )


class TitleColumn(LinkColumn):
    grok.baseclass()
    header = PMF("Title")
    weight = 10

    def getLinkContent(self, item):
        title = get_value(item, 'Title')
        return escape(safe_unicode(title))


class IconColumn(LinkColumn):
    grok.baseclass()

    def getLinkContent(self, item):
        content = super(IconColumn, self).getLinkContent(item)  # escaped
        return u"""<img title="%s" src="%s" />""" % (
            content,
            '%s/%s' % (self.table.portal_url, self.iconName))


class DeleteColumn(IconColumn):
    grok.baseclass()
    header = u""
    weight = 9
    linkName = "delete_confirmation"
    linkContent = PMF('Delete')
    linkCSS = 'edm-delete-popup'
    iconName = "delete_icon.png"
    linkContent = PMF(u"Delete")

    def actionAvailable(self, item):
        obj = item.getObject()
        sm = getSecurityManager()
        return sm.checkPermission('Delete objects', obj)

    def renderCell(self, item):
        if not self.actionAvailable(item):
            return u""

        return super(DeleteColumn, self).renderCell(item)


class DownloadColumn(IconColumn):
    grok.baseclass()
    header = u""
    weight = 1
    linkName = "@@download"
    iconName = "download_icon.png"
    linkContent = _(u"Download file")


class ExternalEditColumn(IconColumn):
    grok.baseclass()
    header = u""
    weight = 3
    linkName = "@@external_edit"
    iconName = "extedit_icon.png"
    linkContent = PMF(u"Edit with external application")

    def actionAvailable(self, item):
        obj = item.getObject()
        sm = getSecurityManager()
        if not sm.checkPermission('Modify portal content', obj):
            return False

        if obj.file is None:
            return False

        ext = os.path.splitext(obj.file.filename)[-1].lower()
        if ext in (u'.pdf', u'.jpg', '.jpeg'):
            return False

        view = getMultiAdapter((obj, self.request), name='externalEditorEnabled')
        if not view.available():
            return False

        return True

    def renderCell(self, item):
        if not self.actionAvailable(item):
            return u""

        return super(ExternalEditColumn, self).renderCell(item)


class EditColumn(IconColumn):
    grok.baseclass()
    header = u""
    weight = 2
    linkName = "edit"
    iconName = "++resource++fade_edit.png"
    linkContent = PMF(u"Edit")
    linkCSS = 'overlay-form-reload'

    def actionAvailable(self, item):
        obj = item.getObject()
        sm = getSecurityManager()
        return sm.checkPermission('Modify portal content', obj)

    def renderCell(self, item):
        if not self.actionAvailable(item):
            return u""

        return super(EditColumn, self).renderCell(item)


class StateColumn(Column):
    grok.baseclass()
    header = PMF(u"State")
    weight = 50

    def renderCell(self, item):
        try:
            wtool = self.table.wtool
            portal_type = get_value(item, 'portal_type')
            review_state = get_value(item, 'review_state')
            if not review_state:
                return u""
            state_title = wtool.getTitleForStateOnType(review_state,
                                                       portal_type)
            return translate(PMF(state_title), context=self.request)
        except WorkflowException:
            return u""


class LabelColumn(Column):
    grok.baseclass()
    attribute = NotImplemented

    def renderCell(self, item):
        value = get_value(item, self.attribute)
        if value is None:
            value = ''
        return value
